﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PV178.Lecture1.Zoo
{
    /// <summary>
    /// Trida zviratek.
    /// </summary>
    abstract class Animal
    {
        private int hunger;


        // Bezna vlastnost (property) zapouzdrujici datovou polozku hunger.
        /// <summary>
        /// Vraci cislo udavajici jak velky ma zviratko hlad.
        /// </summary>
        public int Hunger
        {
            get { return hunger; }
            protected set
            {
                if (value < 0)
                    throw new ArgumentOutOfRangeException("value");
                hunger = value;
            }
        }

        // Automaticka vlastost (Automatic Property): prekladac si automaticky vytvori 
        // datovou polozku typu string pro uchovani hodnoty a vygeneruje odpovidajici 
        // kod pro get a set.
        /// <summary>
        /// Vraci nebo nastavuje poznamky ke zviratku.
        /// </summary>
        public string Notes { get; set; }

        // Name je automaticka vlastnost "jen" ke cteni. Stale musi mit set, 
        // ale ten je diky modifikatoru private viditelny jen z tridy Animal.
        /// <summary>
        /// Vraci jmeno zviratka. Nase zviratka si totiz na sve jmeno po svem narozeni
        /// zvyknou tak moc, ze uz nidky neslysi na jine jmeno.
        /// </summary>
        public string Name { get; private set; }

        // Vlastnost AnimalKind je abstraktni. 
        /// <summary>
        /// Abstraktni vlastnost vracejici cesky nazev druhu zviratka.
        /// </summary>
        public abstract string AnimalKind { get; }



        /// <summary>
        /// Vytvori nove zviratko se zadanym jmenem. Jmeno musi byt neprazdy retezec.
        /// </summary>
        /// <param name="name">Jmeno zviratka.</param>
        /// <exception cref="ArgumentNullException">Zadane jmeno bylo <c>null</c>.</exception>
        /// <exception cref="ArgumentException">Zadane jmeno byl prazdy retezec nebo obsahovalo jen netisknutelne znaky.</exception>
        public Animal(string name)
        {
            if (name == null)
                throw new ArgumentNullException("name");
            if (string.IsNullOrWhiteSpace(name))
                throw new ArgumentException("Name can't be empty nor consists only of whitespace characters.", "name");

            Name = name;
        }

        /// <summary>
        /// Zviratko bude jist dokud bude mit hlad.
        /// </summary>
        public void EatUntilNotHungry()
        {
            while (Hunger > 0)
            {
                EatMeal();
            }
        }

        /// <summary>
        /// Nechame zviratko najist se u misky nebo neco podobneho.
        /// </summary>
        public virtual void EatMeal()
        {
            if (Hunger > 0)
            {
                Hunger--;
            }
        }

        /// <summary>
        /// Vyvenceni zviratka.
        /// </summary>
        public virtual void Walk()
        {
            Hunger++;
        }

        public override string ToString()
        {
            return string.Format("Zviratko {0}, druh: {3}, hlad: {2}, poznamka: '{1}'.", Name, Notes, Hunger, AnimalKind);
        }
    }
}
